import { asApi } from "renraku/x/identities/as-api.js";
import { authApi } from "../../../features/auth/auth-api.js";
import { videosApi } from "../../../features/videos/api/videos-api.js";
import { exampleApi } from "../../../features/example/api/example-api.js";
import { questionsApi } from "../../../features/questions/api/questions-api.js";
import { livestreamApi } from "../../../features/livestream/api/livestream-api.js";
import { prepareAuthPolicies } from "../../../features/auth/policies/prepare-auth-policies.js";
import { makeAdministrativeApi } from "../../../features/administrative/api/administrative-api.js";
import { standardNicknameGenerator } from "../../../features/auth/utils/nicknames/standard-nickname-generator.js";
export async function assimilateApi({ config, rando, database, dacastSdk, sendLoginEmail, signToken, verifyToken, }) {
    const generateNickname = standardNicknameGenerator({ rando });
    const authPolicies = prepareAuthPolicies({
        config,
        verifyToken,
        appTables: database.apps,
        authTables: database.auth,
    });
    return asApi({
        auth: authApi({
            rando,
            config,
            authPolicies,
            signToken,
            verifyToken,
            sendLoginEmail,
            generateNickname,
        }),
        administrative: makeAdministrativeApi({
            config,
            authPolicies,
        }),
        questions: questionsApi({
            rando,
            config,
            authPolicies,
            questionsTables: database.questions,
        }),
        example: exampleApi({
            rando,
            config,
            authPolicies,
            exampleTables: database.example,
        }),
        livestream: livestreamApi({
            authPolicies,
            livestreamTables: database.livestream,
        }),
        videos: videosApi({
            config,
            dacastSdk,
            authPolicies,
            videoTables: database.videos,
        }),
    });
}
//# sourceMappingURL=assimilate-api.js.map