import { apiContext } from "renraku/x/api/api-context.js";
import { findAll } from "../../../../toolbox/dbby/dbby-helpers.js";
import { validateShowLabel } from "../validation/livestream-validators.js";
import { each, validator, schema, minLength, maxLength, array } from "../../../../toolbox/darkvalley.js";
import { runValidation } from "../../../../toolbox/topic-validation/run-validation.js";
import { DamnId } from "../../../../toolbox/damnedb/damn-id.js";
export const makeLivestreamViewingService = ({ authPolicies, livestreamTables, }) => apiContext()({
    policy: async (meta, request) => {
        const auth = await authPolicies.userPolicy(meta, request);
        auth.checker.requirePrivilege("view livestream");
        const appId = DamnId.fromString(auth.access.appId);
        return {
            ...auth,
            livestreamTables: livestreamTables.namespaceForApp(appId),
        };
    },
    expose: {
        async getShows({ livestreamTables }, inputs) {
            if (inputs.labels.length === 0)
                return [];
            const { labels } = runValidation(inputs, schema({
                labels: validator(array(), minLength(1), maxLength(100), each(validateShowLabel)),
            }));
            const shows = await livestreamTables.shows.read(findAll(labels, label => ({ label })));
            return labels.map(label => {
                return shows.find(show => show.label === label)
                    ?? { label, vimeoId: "" };
            });
        }
    },
});
//# sourceMappingURL=livestream-viewing-service.js.map