import { isPermittedToView } from "./is-permitted-to-view.js";
import { findAll } from "../../../../../toolbox/dbby/dbby-helpers.js";
export async function getVideoViews({ labels, videoTables, checker, userPrivileges, }) {
    if (labels.length === 0)
        return [];
    const dacastRows = await videoTables.viewDacast.read(findAll(labels, label => ({ label })));
    const privilegeRows = await videoTables.viewPrivileges.read(findAll(labels, label => ({ label })));
    return dacastRows.map(({ label, type, dacastId }) => {
        const privileges = privilegeRows
            .filter(p => p.label === label)
            .map(p => p.privilegeId.toString());
        const hasExplicitPrivilege = isPermittedToView({
            viewPrivileges: privileges,
            userPrivileges,
        });
        const isPermitted = hasExplicitPrivilege
            || checker.hasPrivilege("view all videos");
        return isPermitted
            ? {
                type,
                label,
                privileges,
                id: dacastId,
                provider: "dacast",
            }
            : undefined;
    });
}
//# sourceMappingURL=get-video-views.js.map